/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "defaults.h"

#include "checksums.h"
#include "stdio.h"
#include "hexdump.h"
#include <netinet/ip.h>
#include <netinet/udp.h>

static __u32 do_sum(__u32 sum, __u8 *buf, int nBytes)
{
   int nleft=nBytes;
   __u16 *w = (__u16*)buf;
   
   while (nleft > 1) {
       sum += *w++;
       nleft -= 2;
   }
   
   if (nleft == 1) {
       __u16 answer = 0;
       *(__u8*)(&answer) = *(__u8*)w;
       sum += answer;
   }
   return sum;
}

static __u16 foldsum(__u32 sum)
{
    while (sum>>16)
        sum = (sum >> 16) + (sum & 0xFFFF);

    return ((unsigned short) ~sum);
}
    
__u16 IPChecksum(iphdr *ipHeader)
{
    return foldsum(do_sum(0, (__u8*) ipHeader, sizeof(iphdr)));
    
}   /* IpChecksum() */


__u16 UDPChecksum(iphdr *ipHeader, udphdr *udpHeader)
{
    __u32 sum = 0;

    // Ip-Pseudo-Header
    sum = do_sum(sum, (__u8*)(&ipHeader->saddr), sizeof(ipHeader->saddr));
    sum = do_sum(sum, (__u8*)(&ipHeader->daddr), sizeof(ipHeader->daddr));
    sum += udpHeader->len;
    sum += ipHeader->protocol<<8;

    sum = do_sum(sum, (__u8*)udpHeader, ntohs(udpHeader->len));

    return foldsum(sum);
}
