/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <sstream>
#include <iomanip>
#include <netinet/in.h>
using namespace std;


#include "defaults.h"
#include "infosatdata.h"
#include "infosatdecoder.h"
#include "hexdump.h"
#include "day.h"

cInfosatDecoder::cInfosatDecoder():
  cMMapFile("infosat.mmap", sizeof(tInfosatMMapHead), "INFOSAT-MMAP 01", 16)
{
  m_mmapHead=(tInfosatMMapHead*)getMMAPData();
  if (m_mmapCreatedNew) {
    m_mmapHead->m_percentFinished=0.0;
    m_mmapHead->m_goodPackets=0;
    m_mmapHead->m_packetCount=1;
  }  
  
  m_startTime=time(NULL);
  m_lastPacketReceivedTime=m_startTime;
  m_packetTimeout=20;
  for (int i=EPG_FIRST_DAY_MAC; i<=EPG_LAST_DAY_MAC; i++) {
    epg(i).initialize(this, i, !m_mmapCreatedNew);
    Attach(&epg(i));
  }
}


cInfosatDecoder::~cInfosatDecoder()
{
}

__u8 infosat_mac[6]	={0x01,0x00,0x5e,0x02,0x02,0x00};
__u8 infosat_mac_mask[6]={0xff,0xff,0xff,0xff,0xff,0x00};

bool cInfosatDecoder::Receive(tMPE& mpe)
{
  bool mac_matched=true;
  for (int i=0;i<6;i++) {
    if ((mpe.m_dest_mac[i] & infosat_mac_mask[i]) != infosat_mac[i]) {
      mac_matched=false;
    }
  }
  if (!mac_matched) {
    return false;
  }
  tInfosat is;
  is.m_mpe=&mpe;
  is.m_header=(infosathdr*)mpe.m_content;
  if (ntohs(is.m_header->TechnisatID) != 0x0001) {
    return false;
  }
  is.m_content=mpe.m_content+sizeof(infosathdr);
  is.m_content_length=mpe.m_content_length-sizeof(infosathdr);
  is.m_mac=mpe.m_dest_mac[5];
  is.m_packetNumber=htons(is.m_header->pktnr);
  is.m_packetCount=htons(is.m_header->pktcount);

  Send(is);  
  return true;
}

void cInfosatDecoder::updateStatistics(void)
{
  m_mmapHead->m_packetCount=0;
  m_mmapHead->m_goodPackets=0;
  for (int i=EPG_FIRST_DAY_MAC; i<=EPG_LAST_DAY_MAC; i++) {
    m_mmapHead->m_packetCount += epg(i).getPacketCount();
    m_mmapHead->m_goodPackets += epg(i).getGoodPackets();
  }
  m_mmapHead->m_percentFinished=double(m_mmapHead->m_goodPackets)/m_mmapHead->m_packetCount * 100;
  if (m_mmapHead->m_packetCount == m_mmapHead->m_goodPackets) {
    deleteMMapFile();
    setExit("infosatdecoder","data reception complete");
  }
  if (abs(time(NULL)-m_lastPacketReceivedTime)>m_packetTimeout) {
    setExit("infosatdecoder","Packet timeout");
  }
}

void cInfosatDecoder::printStatus(ostream &os)
{
  os << "Received " << setw(4) << m_mmapHead->m_goodPackets << " of " << setw(4) << m_mmapHead->m_packetCount << " data blocks [";
  os.setf(ios::fixed);
  os << setw(6) << setprecision(2) << m_mmapHead->m_percentFinished << "%]" << endl;
  for (int i=EPG_FIRST_DAY_MAC; i<=EPG_LAST_DAY_MAC; i++) {
    epg(i).printStatus(os);
  }
}

void cInfosatDecoder::resetEPG(int mac, int day, int month)
{
  cDay d(day, month);
  d -= mac;
  for (int i=EPG_FIRST_DAY_MAC; i<=EPG_LAST_DAY_MAC; i++) {
    d++;
    epg(i).reset(d.m_day, d.m_month);
  }
}

tInfosatMMapStream* cInfosatDecoder::getEPGStreamStatus(int mac)
{
  return &(m_mmapHead->m_streams[mac-EPG_FIRST_DAY_MAC]);
}

void cInfosatDecoder::setPacketReceived()
{
  m_lastPacketReceivedTime=time(NULL);
}

