/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <sstream>
#include <iomanip>
#include <fcntl.h>
using namespace std;

#include "infosatstream.h"
#include "infosatdecoder.h"
#include "infosatdata.h"
#include "hexdump.h"

int cInfosatStream::last_mac=0;
int cInfosatStream::last_packet=0;

cInfosatStream::cInfosatStream()
{
  m_mac=-1;
  m_decoder=NULL;
  m_status=NULL;
  m_keepData=false;
  m_output_fd=-1;
}

cInfosatStream::~cInfosatStream()
{
}

int packetCount_init[EPG_LAST_DAY_MAC+1]={0, 400, 380, 320, 100, 100, 100, 100};

void cInfosatStream::initialize(class cInfosatDecoder *decoder, int mac, bool keepData)
{
  m_decoder=decoder;
  m_mac=mac;
  m_status=m_decoder->getEPGStreamStatus(m_mac);
  m_keepData = keepData;
  if (!m_keepData) {
    reset(0,0);
  }
}

bool cInfosatStream::Receive(tInfosat& is)
{
  if (is.m_mac!=m_mac)
    return false;

  int tmp_day=is.m_header->data[0];
  int tmp_month=is.m_header->data[1];
  
  // Reset all if I get other than expected day
  if ((m_status->m_day != tmp_day) || (m_status->m_month != tmp_month))
    m_decoder->resetEPG(m_mac, tmp_day, tmp_month);


  // If packetCount is other than expected ...  
  if (is.m_packetCount != m_status->m_packetCount) {
    if (!isEmpty()) {
      // Reset if there is already data
      reset(tmp_day, tmp_month);
    }
    // or only set packetCount to real value
    m_status->m_packetCount = is.m_packetCount;
  }

  last_mac=m_mac;
  last_packet=is.m_packetNumber;
  m_decoder->setPacketReceived();
  
  if (isFinished())
    return true;

  openFile();
  
  int packetSize=1400;
  lseek(m_output_fd,is.m_packetNumber*packetSize, SEEK_SET);
  write(m_output_fd,is.m_content, is.m_content_length);
//  m_output.flush();
  
  if (m_status->m_blocks[is.m_packetNumber]==0) {
    m_status->m_goodPackets++;
    m_status->m_blocks[is.m_packetNumber]=1;
    if (m_status->m_goodPackets == m_status->m_packetCount) {
      closeFile();
    }
  }
}

void cInfosatStream::reset(int day, int month)
{
  closeFile();
  // truncate file
  m_keepData=false;
  memset(m_status,0,sizeof(*m_status));
  m_status->m_day=day;
  m_status->m_month=month;
  m_status->m_goodPackets=0;
  // initialize PacketCount
  m_status->m_packetCount=packetCount_init[m_mac];
}

void cInfosatStream::openFile(void)
{
  if (m_output_fd != -1)
    return;
  
  ostringstream fnameStream;
  fnameStream << output_dir << "/infosat_" << setfill('0') << setw(2) << m_status->m_month << "_" << setw(2) << m_status->m_day;
  m_filename=fnameStream.str();
  m_output_fd=open(m_filename.c_str(), O_CREAT | O_RDWR | (m_keepData ? 0 : O_TRUNC), 0644);
}

void cInfosatStream::closeFile(void)
{
  if (m_output_fd != -1) {
    close(m_output_fd);
    m_output_fd=-1;
  }
}

void cInfosatStream::printStatus(ostream &os)
{
  os << m_mac << " ";
  if (isDateEmpty()) {
    os << "EMPTY" << endl;
    return;
  }
    
  os << "DAY " << setfill('0') << setw(2) << m_status->m_day << "." << setw(2) << m_status->m_month << "  " << setfill(' ');
  if (m_status->m_goodPackets==0) {
    os << "[---/" << m_status->m_packetCount << "]";
  } else if (m_status->m_goodPackets == m_status->m_packetCount) {
    os << "[   done]";
  } else {
    os << "[" << setw(3) << m_status->m_goodPackets << "/" << setw(3) << m_status->m_packetCount << "]";
  }
      
    if (verbose_level>=1) {
      string status="";
      char out='0';
      int good=0;
      int pkts=0;
      bool hitDownload=false;
      for (int i=0; i<m_status->m_packetCount; i++) {
        pkts++;
	if ((m_mac==last_mac) && (i==last_packet))
	  hitDownload=true;
        if (m_status->m_blocks[i])
          good++;
        if (hitDownload) {
	  out='~';
	} else if (good==0) {
          out='_';
        } else if (good==pkts) {
          out='#';
        } else {
          out=('0'+good);
        }
	if (pkts == 8) {
          status += out;
	  pkts=0;
	  good=0;
	  hitDownload=false;
        }
      }
      if (pkts) {
        status += out;
      }
      os << "  " << status;
    }
  os << endl;
}
