/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "defaults.h"

#include <stdio.h>
#include <stdlib.h>

#include <iostream>
#include <sstream>
#include <string>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>

#ifdef HAVE_LIBNCURSES
#include <ncurses.h>
#endif

#include "sectiondecoder.h"
#include "sectionfile.h"
#include "sectiondevice.h"
#include "mpedecoder.h"
#include "infosatdecoder.h"
#include "sectiondump.h"

using namespace std;


const char *output_dir=NULL;
int verbose_level=1;
int SysLogLevel=3;

cInfosatDecoder* infosat;



void print_help()
{
	cout
	<< "Usage:" << endl
	<< "	-a #     Number of dvb card to use (0,1,...)" << endl
	<< "	-d #     Number of demux device to use (normally 0)" << endl
	<< "	-c #     Channel to switch vdr to (example viva)" << endl
	<< "	-o dir   Write output-files here" << endl
	<< "	-h       Display this help" << endl
	<< "	-O file  dumps the sections to file" << endl
	<< "	-I file  read sections from file" << endl
	<< "	-q       More quiet output" << endl
	<< "	-v       More verbose output" << endl
	<< endl
	<< "Example:" << endl
	<< "	infosatepg -o /.../mytempdir -c viva -a0" << endl
	<< endl;
}

void sighandler(int);

#ifdef HAVE_LIBNCURSES
static bool has_terminal=true;
#endif

void start_output(void)
{
#ifdef HAVE_LIBNCURSES
	has_terminal=isatty(1);
	if (has_terminal) {
		initscr();
	}
#endif
	signal(SIGTERM, &sighandler);
	signal(SIGINT, &sighandler);
}

void end_output(void)
{
#ifdef HAVE_LIBNCURSES
	if (has_terminal) {
		endwin();
	}
#endif
}


static bool want_exit=false;
static string exit_modul="";
static string exit_comment="";

void setExit(char* modul, char* comment)
{
	exit_modul=modul;
	exit_comment=comment;
	want_exit=true;
}

void doExit()
{
	end_output();
	infosat->updateStatistics();
	infosat->printStatus(cout);
	cout << endl << endl << exit_modul << ": " << exit_comment << endl;
}

void sighandler(int)
{
	setExit("SignalHandler","Kill");
}


/**
 * 
 * @param argc 
 * @param argv 
 * @return 
 */
int main(int argc, char **argv)
{
	cout << "infosatEPG v" << VERSION << endl << endl;
	int adapter=0;
	int demux=0;
	int pid=1809;
	//pid=0x045a;
	char *channel=NULL;
	char *output_file=NULL;
	char *input_file=NULL;
	bool complete_stream=false;

	char opt;
	
	while ((opt = getopt(argc, argv, "ha:d:c:o:O:I:qv")) != -1) {
		switch (opt) {
		case 'a':
			adapter = atoi(optarg);
			break;
		case 'd':
			demux = atoi(optarg);
			break;
		case 'c':
			channel = optarg;
			break;
		case 'o':
			output_dir = optarg;
			break;
		case 'O':
			output_file = optarg;
			break;
		case 'I':
			input_file = optarg;
			break;
		case 'q':
			verbose_level--;
			break;
		case 'v':
			verbose_level++;
			break;
		case 'h':
			print_help();
			exit(0);
			break;	
		}
	}

	struct stat stat_buf;

	if (!output_dir) {
		cerr << "You need to set the output directory!" << endl;
		print_help();
		exit(1);
	}
	if (stat(output_dir, &stat_buf)!=0) {
		perror("output directory");
		print_help();
		exit(1);
	}

	

	cSectionDecoder *sectionDecoder;
	if (input_file) {
		sectionDecoder=new cSectionFile(input_file);
	} else {
		if (channel) {
			char cmd[100];
			snprintf(cmd,100,"svdrpsend.pl chan %s",channel);
			system(cmd);
			sleep(1);
		}
		sectionDecoder=new cSectionDevice(adapter, demux, pid);
	}

	cMPEDecoder mpeDecoder;
	sectionDecoder->Attach(&mpeDecoder);

	infosat=new cInfosatDecoder();
	mpeDecoder.Attach(infosat);
	
	cSectionDump *dumper=NULL;

	if (output_file) {
		dumper=new cSectionDump(output_file);
		sectionDecoder->Attach(dumper);
	}

	sectionDecoder->Start();

	sleep(1);
	
	start_output();	
	while(!want_exit) {
		infosat->updateStatistics();
#ifdef HAVE_LIBNCURSES
		if (has_terminal) {
			clear();
			ostringstream os;
			infosat->printStatus(os);
			addstr(os.str().c_str());
			refresh();
		} else {
#endif
			cout << endl;
			infosat->printStatus(cout);
			cout << endl;
#ifdef HAVE_LIBNCURSES
		}
#endif
		sleep(2);
	}
	doExit();
}
