/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <netinet/ip.h>
#include <netinet/udp.h>
#include <iostream>
using namespace std;

#include "mpedecoder.h"
#include "checksums.h"
#include "hexdump.h"

#define DSMCC_SECTION_MPE 0x3E

cMPEDecoder::cMPEDecoder()
{
}


cMPEDecoder::~cMPEDecoder()
{
}

bool cMPEDecoder::Receive(tSection& s)
{
  if (s.table_id != DSMCC_SECTION_MPE) {
    return false;
  }
  // Test SNAP-Flag
  if ((s.data[5] & 2) != 0) {
    return false;
  }
  tMPE mpe;
  mpe.m_section=&s;
  mpe.m_dest_mac[0]=s.data[11];
  mpe.m_dest_mac[1]=s.data[10];
  mpe.m_dest_mac[2]=s.data[9];
  mpe.m_dest_mac[3]=s.data[8];
  mpe.m_dest_mac[4]=s.data[4];
  mpe.m_dest_mac[5]=s.data[3];
  
  mpe.m_iphdr=(iphdr*)s.content;
  
  // Only IPv4
  if (mpe.m_iphdr->version != 4) {
    return false;
  }

  // Test IP-Checksum  
  if (IPChecksum(mpe.m_iphdr) != 0)
    return false;

  // Only UDP at the moment
  if (mpe.m_iphdr->protocol != 0x11) {
    return false;
  }
  
  mpe.m_udphdr=(udphdr*)(s.content+sizeof(iphdr));
  
  int len_from_ip=ntohs(mpe.m_iphdr->tot_len);
  int len_from_udp=ntohs(mpe.m_udphdr->len);
  if (len_from_ip != len_from_udp+sizeof(iphdr))
    return false;

  mpe.m_content = s.content+sizeof(iphdr)+sizeof(udphdr);
  mpe.m_content_length=len_from_udp-sizeof(udphdr);
  
  if (UDPChecksum(mpe.m_iphdr, mpe.m_udphdr) != 0) {
    return false;
  }

  return Send(mpe);
}

