/***************************************************************************
 *   Copyright (C) 2004-2005 by Matthias Schwarzott                        *
 *   zzam@gmx.de                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <linux/dvb/dmx.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>

#include "defaults.h"
#include "sectiondevice.h"

using namespace std;

cSectionDevice::cSectionDevice(int _adapter, int _demux, int _pid)
{
    adapter  = _adapter;
    demux    = _demux;
    pid      = _pid;
    demux_fd = -1;
}

cSectionDevice::~cSectionDevice()
{
	Close();
}


bool cSectionDevice::Open()
{
	struct dmx_sct_filter_params sctFilterParams;
	char demuxdevice[100];
	snprintf(demuxdevice, 100, "/dev/dvb/adapter%d/demux%d", adapter, demux);

	if ((demux_fd = open(demuxdevice, O_RDWR)) < 0) {
		perror("open demux DEVICE:");
		return false;
	}
	memset(&sctFilterParams, 0, sizeof(sctFilterParams));

	sctFilterParams.pid = pid;
	sctFilterParams.flags = DMX_IMMEDIATE_START | DMX_CHECK_CRC;
	sctFilterParams.filter.filter[0] = 0x3e;
	sctFilterParams.filter.mask[0] = 0xff;

	// Size of read packets 1456
	const int bufsize = 1456*2048;

	if (ioctl(demux_fd, DMX_SET_BUFFER_SIZE, bufsize)<0) {
		perror("DMX SET BUFFER SIZE");
		close(demux_fd);
		demux_fd=-1;
		return false;
	}
	
	if (ioctl(demux_fd, DMX_SET_FILTER, &sctFilterParams) < 0) {
		perror("fd_sect - DMX_SET_FILTER:");
		close(demux_fd);
		demux_fd=-1;
		return false;
	}
	return true;
}

void cSectionDevice::Close()
{
	if (demux_fd!=-1)
	{
		close(demux_fd);
		demux_fd=-1;
	}
}

void cSectionDevice::Action()
{
	__u8 sectbuffer[4096];
	int size;
	Open();
	cerr << "Start receiving from PID " << pid << endl;
	while (1) {
		int size=read(demux_fd, sectbuffer, sizeof(sectbuffer));
		if (size>0) {
			Process(sectbuffer,size);
		} else {
			usleep(100);
		}
	}
	Close();
}
